import gym

from core.config import BaseConfig
from collections import defaultdict

ENVS = ['LunarLanderContinuous-v2', 'BipedalWalker-v3', 'BipedalWalkerHardcore-v3']
ACTION_REPEATS = defaultdict(lambda: 2)


class Box2DConfig(BaseConfig):
    def __init__(self):
        super(Box2DConfig, self).__init__(max_env_steps=int(5e5),
                                          start_step=int(5000),
                                          replay_memory_capacity=int(5e5),
                                          env_itr_steps=500,
                                          valid_envs=ENVS,
                                          action_repeats=ACTION_REPEATS,
                                          action_repeat_set=[1, 2, 4, 8, 16],
                                          policy_lr=1e-4,
                                          value_lr=1e-4,
                                          dynamics_lr=1e-3,
                                          test_interval_steps=10000)

    def new_game(self, seed=None):
        env = gym.make(self.env_name)
        if seed is not None:
            env.seed(seed)
        return env


run_config = Box2DConfig()
